(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define('ultra', ['exports'], factory) :
	(factory((global.ultra = global.ultra || {})));
}(this, (function (exports) { 'use strict';

function id(x) {
  return x;
}

const strProto = Object.getPrototypeOf('');
function isStr(s) {
  return Object.getPrototypeOf(Object(s)) === strProto;
}

function empty(t) {
  return !t || !t.length && !Object.keys(t).length;
}

function makeArray(arr) {
  return Array.isArray(arr) ? arr : empty(arr) ? [] : [arr];
}

function pipe(...fns) {
  function invoke(v) {
    return fns.reduce((acc, fn) => fn ? fn.call(this, acc) : acc, v);
  }
  return fns.length > 0 ? invoke : id;
}

const flattenToObj = (arr, base = {}) => Object.assign(base, ...arr);

function exclude(t, ...keys) {
  return flattenToObj(Object.keys(t).filter(k => keys.indexOf(k) === -1).map(k => ({ [k]: t[k] })));
}

function substitute(literals, values, removeEmpty) {
  let vals = Array.from(values, v => v || '');
  let lits = Array.from(literals, v => v || '');
  if (removeEmpty && lits.length > vals.length) {
    lits = [lits[0], ...lits.slice(1).map((l, i) => l || (vals[i] = ''))];
  }
  return String.raw({ raw: lits }, ...vals);
}

function escapeRx(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
}

function warnOn(warnCheck, msg) {
  return (/*@__PURE__*/warnCheck() && console.error(msg)
  );
}
let $devWarnOn = function () {};
if (typeof process !== 'object' || process.env.NODE_ENV !== 'production') {
  $devWarnOn = warnOn;
}

class Timer {
  constructor(cb, wait, ms = 0) {
    this.run = this.run.bind(this, cb, ms);
    if (!wait) this.run();
  }
  get active() {
    return !!this.ref;
  }
  run(cb, ms) {
    return this.ref = setTimeout(this.stop.bind(this, cb), ms);
  }
  stop(cb) {
    clearTimeout(this.ref);
    this.ref = undefined;
    return cb && cb();
  }
}

let isTimer = timer => timer && timer instanceof Timer ? timer : false;
let scheduleTask = (...args) => new Timer(...args);

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();











var toArray = function (arr) {
  return Array.isArray(arr) ? arr : Array.from(arr);
};

let urlx = /^http.?:[\/]{2}[^\/]+/;

function stripPrefix(prefix, path) {
  return prefix ? path.replace(new RegExp('^' + escapeRx(prefix)), '') : path;
}

function addLeadingSlash(path) {
  return path ? path.replace(/^([?#]|[^/])/, '/$1') : '/';
}

function removeTrailingSlash(path) {
  return path === '/' ? path : path.replace(/\/$/, '');
}

function normalizePath(prefix) {
  return pipe(stripPrefix.bind(null, prefix), addLeadingSlash, removeTrailingSlash);
}

function encodePath(path, decoded) {
  if (!decoded) path = decode(path);
  return encodeURI(path).replace(/%5B/g, '[').replace(/%5D/g, ']');
}

function encodeData(data, decoded) {
  if (!decoded) data = decode(data);
  return encodeURIComponent(data);
}

function decode(path) {
  let result;
  try {
    result = decodeURIComponent(path);
  } catch (ex) {
    console.error('Error: Could not decode path:', path);
    result = '';
  }
  return result;
}

function verifyHashEncoding(h) {
  return !h || encodeData(h) === h;
}

function extractHash(loc) {
  var _loc$split = loc.split(/#(.*)$/, 2),
      _loc$split2 = slicedToArray(_loc$split, 2);

  let locwof = _loc$split2[0],
      h = _loc$split2[1];

  /*@__PURE__*/$devWarnOn(() => !verifyHashEncoding(h), 'Use encodeURIComponent to encode fragment data: ' + h);
  return [locwof, h];
}

function verifyQSEncoding(qs) {
  if (qs) qs = qs.replace(/[=&,]/g, '');
  return !qs || encodeData(qs) === qs;
}

function extractQS(loc) {
  var _loc$split3 = loc.split(/\?(.*)$/, 2),
      _loc$split4 = slicedToArray(_loc$split3, 2);

  let path = _loc$split4[0],
      qs = _loc$split4[1];

  /*@__PURE__*/$devWarnOn(() => !verifyQSEncoding(qs), 'Use encodeURIComponent to encode query string data: ' + qs);
  return [path, qs];
}

function extractQSHash(loc) {
  var _extractHash = extractHash(loc),
      _extractHash2 = slicedToArray(_extractHash, 2);

  let locwof = _extractHash2[0],
      h = _extractHash2[1];

  var _extractQS = extractQS(locwof),
      _extractQS2 = slicedToArray(_extractQS, 2);

  let path = _extractQS2[0],
      qs = _extractQS2[1];

  return [path, qs, h];
}

function extractPath(path) {
  return path ? encodePath(path.replace(urlx, '')) : '';
}

function makeLocation(path, qs, hash) {
  path = extractPath(path);
  let href = substitute([path, qs, hash], ['?', '#'], true);
  return Object.assign({ path, href }, qs && { qs }, hash && { hash });
}

function parseHref(loc) {
  return makeLocation(...extractQSHash(loc));
}

function parseQS(qs, ids, { delim = ',', defaults: defaults$$1 = [], decodeValues } = {}) {
  if (qs[0] !== '?') qs = '?' + qs;
  let values = ids.map((id, i) => {
    let rx = new RegExp('[?&;]+' + escapeRx(id) + '=([^&;#]+)', 'i');
    let res = qs.split(rx).slice(1).filter(s => /^[^&;#]/.test(s));
    res = (decodeValues ? res.map(decode) : res).join(delim);
    return res || defaults$$1[i] || '';
  });
  return values;
}

function prependPath(values, path) {
  let slashes = new Array(values.length).fill('/', path.slice(-1) === '/' ? 1 : 0);
  return substitute([path, ...values], slashes);
}

function appendPath(values, path) {
  let slashes = new Array(values.length).fill('/', 0, path.slice(0, 1) === '/' ? -1 : void 0);
  return substitute([...values, path], slashes);
}

let env = {
  get window() {
    /*@__PURE__*/$devWarnOn(() => typeof window === 'undefined', 'missing window object in environment');
    return window;
  },
  get location() {
    return this.window.location;
  },
  get path() {
    return removeTrailingSlash(this.location.pathname);
  },
  get qs() {
    return this.location.search.slice(1);
  },
  get hash() {
    return this.location.hash.slice(1);
  },
  get href() {
    return substitute([this.path, this.qs, this.hash], ['?', '#'], true);
  },
  get origin() {
    var _location = this.location;
    let protocol = _location.protocol,
        hostname = _location.hostname,
        port = _location.port;

    return substitute([protocol, hostname, port], ['//', ':'], true);
  },
  get history() {
    return this.window.history;
  }
};

const literalp = '([^\\s/]*)';
const allx = /(?:)/;
const identifierx = /(:[A-Za-z0-9_:]+)/;

function getMatchX(identifiers, literals) {
  let subs = new Array(identifiers.length).fill(literalp);
  return new RegExp('^' + substitute(literals.map(escapeRx), subs), 'i');
}

let parsePathKey = pathKey => {
  let key = isStr(pathKey) ? removeTrailingSlash(pathKey) : '';
  let fragments = key.split(identifierx);
  let identifiers = [];
  let literals = fragments.reduce((acc, f) => {
    if (f.indexOf(':') === 0) identifiers.push(f);else acc.push(f);
    return acc;
  }, []);
  let matchx = key == '' ? allx : getMatchX(identifiers, literals);
  return { key: pathKey, identifiers, literals, matchx };
};

function assignValues(pathKey, values = []) {
  var _ref = isStr(pathKey) ? parsePathKey(pathKey) : pathKey;

  let identifiers = _ref.identifiers;

  return flattenToObj(identifiers.map((id, key) => ({ [id]: values[key] })));
}

class Path {
  constructor(key) {
    Object.assign(this, parsePathKey(key));
  }
  findInvalid(checks, values) {
    let ids = this.identifiers,
        hasCheck = Object.prototype.hasOwnProperty.bind(checks);
    let callCheck = (id, val) => hasCheck(id) && !checks[id](val, values, ids);
    return empty(checks) ? -1 : values.findIndex((val, i) => callCheck(ids[i], val));
  }
  validate(checks, values) {
    let invalid = this.findInvalid(checks, values);
    return invalid === -1 ? { values, passed: true } : { values: values.slice(0, invalid), exact: false };
  }
  match(checks, path) {
    let matches = this.matchx.exec(path);
    if (!matches) return {};
    let match = matches[0],
        values = matches.slice(1).map(decode);
    let exact = match.length === path.length,
        ids = this.identifiers;
    return Object.assign({ ids, match, exact }, this.validate(checks, values));
  }
}

function trimIdsValues(sourceIds, targetIds, targetValues) {
  let stopReplaceIndex = sourceIds.findIndex((sid, i) => targetIds[i] !== sid);
  if (stopReplaceIndex === -1) stopReplaceIndex = sourceIds.length;
  return [targetIds.slice(stopReplaceIndex), targetValues.slice(stopReplaceIndex)];
}

class PathSpec {
  constructor(pathKeys, next, err, fail) {
    if (!Array.isArray(pathKeys) || !pathKeys.length) pathKeys = [isStr(pathKeys) ? pathKeys : ''];
    let paths = pathKeys.map(k => new Path(k));
    Object.assign(this, { pathKeys, paths, next, err, fail });
  }
  find(pathKey) {
    let idx = this.pathKeys.indexOf(pathKey);
    return idx > -1 && this.paths[idx];
  }
  match(checks, path) {
    var _paths = toArray(this.paths);

    let primary = _paths[0],
        subs = _paths.slice(1);

    let result,
        matches = primary.match(checks, path);
    if (matches.passed) {
      result = {
        ids: matches.ids,
        values: matches.values,
        [primary.key]: exclude(matches, 'passed')
      };
      if (!matches.exact) {
        subs.some(sub => {
          let submatches = sub.match(checks, path);
          if (submatches.passed) {
            var _trimIdsValues = trimIdsValues(result.ids, submatches.ids, submatches.values),
                _trimIdsValues2 = slicedToArray(_trimIdsValues, 2);

            let ids = _trimIdsValues2[0],
                vals = _trimIdsValues2[1];

            result.ids = result.ids.concat(ids);
            result.values = result.values.concat(vals);
            result[sub.key] = exclude(submatches, 'passed');
          }
          return submatches.exact;
        });
      }
    }
    return result;
  }
  success(result) {
    return result && Object.keys(result).some(k => result[k].exact);
  }
  resolve(result, success = this.success(result)) {
    /*@__PURE__*/$devWarnOn(() => !success && !this.err, 'Resolve location with partial match w/o dedicated handler: ' + (result && result.href));
    return !this.err || success ? this.next(result) : this.err(result);
  }
  reject(result) {
    return this.fail && this.fail(result);
  }
}

function spec(...pathKeys) {
  return (next, err, fail) => new PathSpec(pathKeys, next, err, fail);
}

class PrefixSpec extends PathSpec {
  get prefixKey() {
    return this.pathKeys[0];
  }
  match(checks, msg) {
    let path = msg.path,
        result = Object.assign({}, msg);
    let matches = super.match(checks, path);
    if (matches) {
      var _matches$prefixKey = matches[this.prefixKey];
      let prefix = _matches$prefixKey.match,
          pValues = _matches$prefixKey.values,
          pIds = _matches$prefixKey.ids;

      result = super.resolve(Object.assign(result, { prefix, pIds, pValues }), true);
    } else result.success = false;
    return result;
  }
}

function prefixSpec(prefixKey, next) {
  return new PrefixSpec(prefixKey, next);
}

function rxFn(rxs) {
  return value => rxs.every(rx => rx.test(value));
}

function makeCheck(id, rxs) {
  return { [id]: rxFn(rxs) };
}

function rx(ids, ...rxs) {
  return flattenToObj(ids.map(id => makeCheck(id, rxs)));
}

function check(...ids) {
  return rx.bind(null, ids);
}

class MissSpec extends PathSpec {
  match(result) {
    let hasCheck = Object.prototype.hasOwnProperty.bind(result);
    let miss = this.paths.filter(({ key }) => !hasCheck(key)).map(({ key }) => key);
    let matched = miss.length === this.paths.length;
    return matched && super.resolve(Object.assign({}, result, { miss }), true);
  }
}

function miss(next, ...pathKeys) {
  return new MissSpec(pathKeys, next);
}

const off = () => false;
const turnOff = o => Object.assign(o, { match: off, resolve: off, reject: off });

function toggle(match, newKey, newMatch) {
  let off = match.off,
      key = match.key;

  let on = off ? newMatch || off : turnOff({ off: newMatch || match });
  on.key = newKey || key;
  let hasAll = (o, ...props) => props.every(Object.prototype.hasOwnProperty.bind(o));
  /*@__PURE__*/$devWarnOn(() => !hasAll(on, 'match', 'resolve', 'reject'), 'Match not well formed: ' + on.key);
  return on;
}

function toggleSelected(matchers, key, replacements) {
  if (key === undefined) return matchers;
  replacements = makeArray(replacements);
  let replaced = 0;
  let result = matchers.map(m => m.key === key ? toggle(m, key, replacements[replaced++]) : m);
  /*@__PURE__*/$devWarnOn(() => replaced === 0 || replaced !== replacements.length, replaced + ' replacements issued for toggle key: ' + key);
  return result;
}

function matcher(specs, checks, msg) {
  let spec$$1,
      result,
      path = msg.path;
  spec$$1 = specs.find(s => !!(result = s.match(checks, path)));
  let success = spec$$1 && spec$$1.success(result);
  result = Object.assign({}, msg, result);
  return { result, success, spec: spec$$1 };
}

function resolveSpec({ result, success, spec: spec$$1 }) {
  let timer = isTimer(spec$$1.resolve(result, success));
  return timer ? { result, timer } : { result };
}

function resolve(msg) {
  return msg.spec ? Object.assign({}, msg, resolveSpec(msg)) : false;
}

function reject(specs, msg) {
  let result = msg.result,
      spec$$1 = msg.spec;

  specs.forEach(s => s !== spec$$1 && s.reject(result));
}

function matchPrefix(prefixKey, matcher) {
  let match = matcher.match,
      checks = matcher.checks;

  let pspec = prefixSpec(prefixKey, match);
  return Object.assign({}, matcher, { prefixKey, match: pspec.match.bind(pspec, checks) });
}

function prematch(matchCheck, msg) {
  let prefix = msg.prefix,
      href = msg.href,
      path = msg.path;
  var _msg$qs = msg.qs;
  let qs = _msg$qs === undefined ? '' : _msg$qs;
  var _msg$hash = msg.hash;
  let hash = _msg$hash === undefined ? '' : _msg$hash;

  if (prefix) path = normalizePath(prefix)(path);
  if (matchCheck) path = matchCheck({ prefix, href, path, qs, hash });
  return path === msg.path ? msg : Object.assign({}, msg, { path });
}

function match(specs, checks = {}, matchCheck) {
  if (!Array.isArray(specs)) specs = [].concat(specs);
  let match = pipe(prematch.bind(null, matchCheck), matcher.bind(null, specs, checks));
  return { match, resolve, specs, checks, reject: reject.bind(null, specs) };
}

function prefixMatch(prefixKey, matcher, prematchCheck) {
  let prefixed = matchPrefix(prefixKey, matcher);
  if (prematchCheck) {
    let match = prefixed.match;

    prefixed.match = pipe(prematch.bind(null, prematchCheck), match);
  }
  return prefixed;
}

function dispatch(actions, msg) {
  let result,
      resolved = actions.some(fn => !!(result = fn(msg)));
  /*@__PURE__*/$devWarnOn(() => !resolved, 'Could not resolve location: ' + msg.href);
  return result;
}

function settle(matchers, mismatchers, msg) {
  let timer = msg.timer,
      result = msg.result;

  if (result && !(isTimer(timer) && timer.active)) {
    if (timer) timer.run();
    matchers.forEach(matcher => matcher.reject(msg));
    mismatchers.forEach(mm => mm.match(result));
  }
}

function makeDispatcher(matchers, mismatchers) {
  let actions = matchers.map(matcher => pipe(matcher.match, matcher.resolve));
  let dispatcher = pipe(dispatch.bind(null, actions), settle.bind(null, matchers, mismatchers));
  return dispatcher;
}

let defaultHandler = handlers => (...args) => handlers().forEach(h => h(...args));

class Listener extends Set {
  constructor(eventKey, target, handler = defaultHandler) {
    super();
    this.values = this.values.bind(this);
    handler = handler(this.values);
    let orphans = new WeakSet(),
        active = false;
    Object.assign(this, { eventKey, target, handler, orphans, active });
  }
  beginListen() {
    if (!this.active) {
      this.target.addEventListener(this.eventKey, this.handler);
      this.active = true;
    }
  }
  stopListen() {
    if (this.active) {
      this.active = false;
      this.target.removeEventListener(this.eventKey, this.handler);
    }
  }
  add(val) {
    this.beginListen();
    super.add.call(this, val);
    this.orphans.delete(val);
    return this.delete.bind(this, val);
  }
  delete(val) {
    this.orphans.add(val);
  }
  clear() {
    this.values().map(v => this.orphans.add(v));
  }
  *[Symbol.iterator]() {
    [...super.values.call(this)].filter(v => this.orphans.has(v)).map(v => super.delete.call(this, v));
    if (super.size === 0) this.stopListen();else yield* super.values.call(this);
  }
  values() {
    return Array.from(this);
  }
  get size() {
    return this.values().length;
  }
}

function invokeHandlers(handlers) {
  function invoke(event, fn) {
    let href = env.href,
        path = env.path,
        qs = env.qs,
        hash = env.hash,
        state = event.state;

    return fn({ href, path, qs, hash, state });
  }
  return event => handlers().forEach(invoke.bind(null, event));
}

function createPopstate() {
  return new Listener('popstate', env.window, invokeHandlers);
}

let push = (success, msg) => {
  let href = msg.href,
      path = msg.path,
      state = msg.state,
      docTitle = msg.docTitle;

  if (href !== env.href) {
    /*@__PURE__*/$devWarnOn(() => encodePath(path) !== path, 'Incorrect encoding. Use encodeURI on path: ' + path);
    env.history.pushState(state, docTitle, href);
    if (success) return success(msg);
  } else /*@__PURE__*/$devWarnOn(() => true, 'Attempt to push location identical to current one: ' + href);
};

let replace = (success, msg) => {
  let href = msg.href,
      path = msg.path,
      state = msg.state,
      docTitle = msg.docTitle;

  /*@__PURE__*/$devWarnOn(() => encodePath(path) !== path, 'Incorrect encoding. Use encodeURI on path: ' + path);
  env.history.replaceState(state, docTitle, href);
  if (success) return success(msg);
};

let go = val => val && env.history.go(val);

const idk = '$id';

function place(n) {
  return [10, 100].find(x => n % x === n) || 100;
}

function makeId(m, n = 0) {
  return m * place(n) + n;
}

function makeVisit(ultra, state) {
  let currLen = env.history.length,
      visited = ultra.visited || [];
  let origId = state && state[idk],
      id,
      newState;

  var _visited = visited,
      _visited2 = toArray(_visited);

  let len = _visited2[0],
      visits = _visited2.slice(1);

  if (!len || currLen > len) {
    id = makeId(currLen);
    visited = [currLen, id];
  } else {
    id = origId || makeId(currLen, visits.length);
    if (!visits.length || id > visits[visits.length - 1]) {
      visited = [currLen, ...visits, id];
    }
  }
  if (origId !== id) newState = Object.assign({}, state, { [idk]: id });
  return { visited, newState };
}

function recalibrate(msg) {
  let ultra = msg.ultra,
      state = msg.state,
      currentLen = env.history.length;

  var _ultra$visited = toArray(ultra.visited);

  let len = _ultra$visited[0],
      visits = _ultra$visited.slice(1),
      delta = 0;

  if (state && state[idk] && currentLen === len) {
    delta = visits[visits.length - 1] === state[idk] ? -1 : 1;
  }
  return delta;
}

function recordVisit(dispatch, msg) {
  var _msg = msg;
  let ultra = _msg.ultra,
      state = _msg.state;

  var _makeVisit = makeVisit(ultra, state);

  let visited = _makeVisit.visited,
      newState = _makeVisit.newState;

  ultra.visited = visited;
  if (newState) {
    msg = Object.assign({}, msg, { state: newState });
    return replace(dispatch, msg);
  }
  return dispatch(msg);
}

function guardDispatch(ultra, dispatch, loc) {
  var _ultra$tapped = slicedToArray(ultra.tapped, 3);

  let len = _ultra$tapped[0],
      tappedHref = _ultra$tapped[1],
      confirm = _ultra$tapped[2],
      href = loc.href;

  let msg = Object.assign({}, loc, { ultra });
  let ok = () => {
    ultra.untap();
    return dispatch(msg);
  };
  let cancel = pipe(recalibrate, go).bind(null, msg);
  if (confirm && len === env.history.length) {
    if (href !== tappedHref) return confirm(ok, cancel, msg);
  } else return ok();
}

function navigate(ultra, dispatch, navAction, loc) {
  loc = isStr(loc) ? parseHref(loc) : loc;
  return guardDispatch(ultra, navAction.bind(null, dispatch), loc);
}

function run(_matchers, _mismatchers, _popstate) {
  let _tapped = [],
      dispatch = recordVisit.bind(null, makeDispatcher(_matchers, _mismatchers));
  let ultra = {
    visited: null,
    get popstate() {
      return _popstate;
    },
    get matchers() {
      return _matchers;
    },
    get mismatchers() {
      return _mismatchers;
    },
    get tapped() {
      return _tapped;
    },
    untap() {
      return _tapped = [];
    },
    tap(cb) {
      return _tapped = [env.history.length, env.href, cb];
    },
    stop: _popstate.add(loc => guardDispatch(ultra, dispatch, loc)),
    nav: (action, loc) => navigate(ultra, dispatch, action, loc),
    push: loc => ultra.nav(push, loc),
    replace: loc => ultra.nav(replace, loc),
    dispatch: () => ultra.nav((cb, msg) => cb(msg), env.href)
  };
  return ultra;
}

function container(matchers, mismatchers, instance, scheduleDispatch) {
  var _ref = instance || {};

  let stop = _ref.stop,
      popstate = _ref.popstate,
      visited = _ref.visited,
      isRunning = Array.isArray(visited);

  if (scheduleDispatch === void 0) scheduleDispatch = !instance || !isRunning;
  if (!popstate) popstate = createPopstate();
  let ultra = run(makeArray(matchers), makeArray(mismatchers), popstate);
  if (stop) stop.call(instance);
  if (isRunning) ultra.visited = visited;
  if (scheduleDispatch) setTimeout(ultra.dispatch);
  return ultra;
}

const originx = new RegExp('^' + escapeRx(env.origin) + '|^(?!https?://)');

function retainQS(retain, currQS) {
  let qs = retain && /\bqs\b/.test(retain) ? env.qs : '';
  return currQS ? substitute([currQS, qs], ['&'], true) : qs;
}

function retainHash(retain, currHash) {
  return retain && /\bhash\b/.test(retain) ? env.hash : currHash;
}

function retainQSHash(retain, loc) {
  let qs = retainQS(retain, loc.qs);
  let hash = retainHash(retain, loc.hash);
  let href = substitute([loc.path, qs, hash], ['?', '#'], true);
  return { href, qs, hash };
}

function verifyClick(e) {
  return !(e.defaultPrevented || e.button !== 0 || e.metaKey || e.altKey || e.ctrlKey || e.shiftKey);
}

function makeClickHandler({ href, state, docTitle, retain }, action) {
  const loc = Object.assign(parseHref(href), { state, docTitle });
  let clickHandler = e => {
    if (verifyClick(e) && originx.test(href)) {
      e.preventDefault();
      action(retain ? Object.assign({}, loc, retainQSHash(retain, loc)) : loc);
    }
  };
  return Object.assign(clickHandler, loc);
}

function getNavAction(retain) {
  return retain && /\bhistory\b/.test(retain) ? 'replace' : 'push';
}

const defaultStyle = { touchAction: 'manipulation', msTouchAction: 'manipulation' };
const ownKeys = ['createElement', 'getUltra', 'style', 'state', 'docTitle', 'retain', 'onClick'];

function anchor(props) {
  var _props = props;
  let href = _props.href,
      createElement = _props.createElement,
      getUltra = _props.getUltra,
      style = _props.style,
      state = _props.state,
      docTitle = _props.docTitle,
      retain = _props.retain;

  props = exclude(props, ...ownKeys);
  let navAction = getNavAction(retain);
  props.onClick = makeClickHandler({ href, state, docTitle, retain }, loc => getUltra()[navAction](loc));
  props.style = Object.assign({}, defaultStyle, style);
  return createElement('a', props);
}

const a = { link: anchor };

exports.spec = spec;
exports.miss = miss;
exports.check = check;
exports.assignValues = assignValues;
exports.match = match;
exports.prefixMatch = prefixMatch;
exports.toggle = toggle;
exports.toggleSelected = toggleSelected;
exports.parseQS = parseQS;
exports.prependPath = prependPath;
exports.appendPath = appendPath;
exports.scheduleTask = scheduleTask;
exports.container = container;
exports.a = a;

Object.defineProperty(exports, '__esModule', { value: true });

})));
